@@ -85,6 +85,10 @@ sub connect_changed_signal {
 	1;
 }
 
+sub get_gtk_tip_widgets {
+	[ shift->get_gtk_parent_widget->entry ];
+}
+
 1;
 
 __END__
@@ -160,7 +164,7 @@ didn't specify a static presets list using the B<presets> attribute.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2004 by Jrn Reder.
+Copyright 2004-2005 by Jrn Reder.
 
 This library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Library General Public License as
